// EEPROM Programmer
// Chris Ward, 1998-2000

// misc.cpp
//  - File load and save functions
//  - Utility functions

#include <stdio.h>
#include <string.h>
//#include <math.h>
#include "misc.h"


bool loadbin(char *filename, unsigned char *buf, unsigned long size)
{
	FILE *file;

	// Open the file.
	if ( (file = fopen(filename, "rb")) == NULL )
	{
    	return false;
	}

	fread(buf, sizeof(unsigned char), size, file);
	fclose(file);
	return true;
}


bool loadhex(char *filename, unsigned char *buf, unsigned long size, unsigned long base)
{
	FILE *file;
	char linebuf[80];
	unsigned short reclen, rectype, data, p;
	unsigned long offset, i;
	unsigned char chksum;
	bool result = true;

	// Open the file.
	if ( (file = fopen(filename, "rt")) == NULL )
	{
    	return false;
	}

	// Read the file
	while (fscanf(file, "%s", &linebuf) != EOF)
	{
		if (linebuf[0] != ':')			// first character of record must be ':'
			continue;
		if (strlen(linebuf) < 11)
		{	// too small to be a valid record
			result = false;
			break;
		}
		reclen = hextodec(linebuf[1], linebuf[2]);
		chksum = reclen;
		if (reclen == 0)
			break;
		offset = (hextodec(linebuf[3], linebuf[4]) * 256) + hextodec(linebuf[5], linebuf[6]);
		chksum += (offset / 256) + (offset & 0xFF);
		rectype = hextodec(linebuf[7], linebuf[8]);
		chksum += rectype;
		if (rectype == 1)		// record type 01 = end of file
			break;
		else if (rectype != 0)	// if not data record (type 00), ignore
			continue;
		if (offset < base || (offset + reclen) > (base + size))
		{	// address out of range
			result = false;
			break;
		}
		else
		{
			if ( strlen(linebuf) < (reclen * 2 + 11) )
			{	// not enough data in record
				result = false;
				break;
			}
			p = 9;
			for (i = offset; i < offset+reclen; i++)
			{
				data = hextodec(linebuf[p], linebuf[p+1]);
				chksum += data;
				buf[i - base] = data;
				p += 2;
			}
		}
		chksum = ~chksum + 1;		// two's complement
		if (hextodec(linebuf[p], linebuf[p+1]) != chksum)
		{	// checksum failed
			result = false;
			break;
		}
	}

	fclose(file);
	return result;
}


bool savebin(char *filename, unsigned char *buf, unsigned long size)
{
	FILE *file;

	// Open the file.
	if ( (file = fopen(filename, "wb")) == NULL )
	{
    	return false;
	}

	fwrite(buf, sizeof(unsigned char), size, file);
	fclose(file);
	return true;
}


bool savehex(char *filename, unsigned char *buf, unsigned long size, unsigned long base)
{
	FILE *file;
	unsigned short reclen;
	unsigned long i, offset;
	unsigned char chksum;

	// Open the file.
	if ( (file = fopen(filename, "wt")) == NULL )
	{
    	return false;
	}

	for (offset = 0; offset < size; offset += 16)
	{
		if (size - offset < 16)
			reclen = size - offset;
		else
			reclen = 16;
		fprintf(file, ":%02X%04X00", reclen, offset + base);
		chksum = reclen + ((offset+base)/256) + ((offset+base)&0xFF);
		for (i = offset; i < offset + reclen; i++)
		{
			chksum += buf[i];
			fprintf(file, "%02X", buf[i]);
		}
		chksum = ~chksum + 1;			// two's complement
		fprintf(file, "%02X\n", chksum);
	}

	fprintf(file, ":00000001FF\n");		// end-of-file record

	fclose(file);
	return true;
}


// setbit
//  sets bit #'bit' in 'num' to 'state' (0/1)
void setbit(unsigned long& num, int bit, bool state)
{
	if (state)
		num |= (1 << bit);				// set bit
	else
		num &= ~(1 << bit);				// clear bit
}


// testbit
//  test value of bit #'bit' in 'num'
//  returns 0/1 (false/true)
bool testbit(unsigned long num, int bit)
{
	return ((num >> bit) & 1);
}


// hextodec
//  convert two ASCII hex digits to an int
//  dig1 is low nybble, i.e. hextodec('1', '0') returns 16
//  returns -1 on error
int hextodec(char dig2, char dig1)
{
	int num;

	if (dig1 >= '0' && dig1 <= '9')
		num = dig1 - 48;
	else if (dig1 >= 'A' && dig1 <= 'F')
		num = dig1 - 55;
	else
		return (-1);

	if (dig2 >= '0' && dig2 <= '9')
		num += (dig2 - 48) * 16;
	else if (dig2 >= 'A' && dig2 <= 'F')
		num += (dig2 - 55) * 16;
	else
		return (-1);

	return num;
}
